/*******************************************************************
 *                  Light Weight Chord Library 1.0                 *
 *  CopyRight(C) Yoshihide Matsumoto IDEON WorkingGroup 2003,2004  *
 *      matsumoto333@yahoo.co.jp (http://www.matchan.mydns.jp)     *
 *                                                                 *
 * Implementation of chord (Distributed Hash Table)                *
 * This program is distributed under GPL                           *
 * Light Weight Chord Library comes with ABSOLUTELY NO WARRANTY.   *
 * This is free software, and you are welcome to redistribute it   *
 * under certain conditions; read `COPYING' for details.           *
 *******************************************************************/


#ifndef __IDLIST
#define __IDLIST

typedef struct _IDLIST {

  int  id;

  struct _IDLIST *prev;
  struct _IDLIST *next;
}IDLIST;

#endif

IDLIST *id_insert(IDLIST **phead, int id);
IDLIST *id_delete_key(IDLIST **phead, int id);
IDLIST *id_delete(IDLIST **phead, IDLIST *pdel);


IDLIST *id_search(IDLIST *plist, int id);
int  id_size(IDLIST *plist);
void id_print(IDLIST **plist);



