/*******************************************************************
 *                  Light Weight Chord Library 1.0                 *
 *  CopyRight(C) Yoshihide Matsumoto IDEON WorkingGroup 2003,2004  *
 *      matsumoto333@yahoo.co.jp (http://www.matchan.mydns.jp)     *
 *                                                                 *
 * Implementation of chord (Distributed Hash Table)                *
 * This program is distributed under GPL                           *
 * Light Weight Chord Library comes with ABSOLUTELY NO WARRANTY.   *
 * This is free software, and you are welcome to redistribute it   *
 * under certain conditions; read `COPYING' for details.           *
 *******************************************************************/


#define M_ACK                1
#define M_ACK_WAIT           2
#define M_ACK_RESET          3
#define M_JOIN             100

#define M_FILE             101
#define M_JOIN_ACCEPT      102
#define M_JOIN_REDIRECT    103

#define M_LOOKUP           200
#define M_LOOKUP_RESPONSE  201

#define M_INSERT           300

#define M_LEAVE            400
#define M_LEAVE_END        401

#define M_MESSAGE          500

#define M_UPDATE_RT        600



#define F_ACK              0
#define F_JOIN             1
#define F_FILE             2
#define F_JOIN_ACCEPT      3

#define F_LOOKUP           4
#define F_LOOKUP_RESPONSE  5

#define F_INSERT           6

#define F_LEAVE            7
#define F_LEAVE_END        8

#define F_UPDATE_RT        9




#define NONE                 0
#define FIND                 1 //LOOKUP_RESPONSE
#define NOT_FIND             2 //LOOKUP_RESPONSE
